@@ -2,7 +2,7 @@ package Log::Defer;
 
 use strict;
 
-our $VERSION = '0.300';
+our $VERSION = '0.310';
 
 use Time::HiRes;
 use Carp qw/croak/;
@@ -194,10 +194,19 @@ Log::Defer - Deferred logs and timers
 
     use Log::Defer;
     use JSON::XS; ## or whatever
+    use Try::Tiny;
 
     sub my_logger_function {
       my $msg = shift;
-      print JSON::XS->new->pretty(1)->encode($msg);
+      
+      my $encoded_msg = try {
+        JSON::XS->new->pretty(1)->encode($msg)
+      }
+      catch {
+        "Failed to JSON encode msg : $_"
+      };
+
+      print $encoded_msg; ## usually you'd append this to a file
     }
 
     my $logger = Log::Defer->new({
@@ -232,7 +241,7 @@ Prints:
 
 I believe a lot of log processing is done too early.
 
-This module helps you defer log processing in two ways:
+This module lets you defer log processing in two ways:
 
 =over 4
 
@@ -242,7 +251,7 @@ Typically this transaction is something like an HTTP request or a cron job. Gene
 
 =item Defer rendering of log messages
 
-Sometimes you don't know how logs should be rendered until long after the message has been written. If you aren't sure what information you'll want to display, or you expect to display logs in multiple different formats, it makes sense to store your logs in a highly structured format so they can be reliably parsed and then processed later.
+Sometimes you don't know how logs should be rendered until long after the message has been written. If you aren't sure what information you'll want to display, or you expect to display logs in different formats, it makes sense to store your logs in a highly structured format so they can be reliably parsed and then processed later.
 
 =back
 
@@ -264,7 +273,7 @@ The simplest use case is outlined in the L<SYNOPSIS>. You create a new Log::Defe
       ...
     } ## <- $logger goes out of scope here so log is written now
 
-There is no need to manually ensure that every possible code path ends up calling your logging routine at the end because perl's reference counting system does that for you (unless you call C<POSIX::_exit>, don't do that).
+There is no need to manually ensure that every possible code path ends up calling your logging routine at the end because perl's reference counting system does that for you (unless you call C<POSIX::_exit> so don't do that).
 
 In an asynchronous application where multiple asynchronous tasks are kicked off concurrently, each task can keep a reference to the logger object and the log writing will be deferred until all tasks are finished.
 
@@ -275,19 +284,19 @@ Log::Defer makes it easy to gather timing information about the various stages o
 
 =head1 STRUCTURED LOGS
 
-Free-form line-based log protocols are probably the most common log formats by far. The formats are usually just happenstance -- whatever happened to be convenient for the programmer.
+Free-form line-based log protocols are probably the most common log formats by far. The "format" is usually just coincidental -- whatever happened to be convenient for the programmer to record.
 
-Unfortunately, doing analysis on ad-hoc unstructured multi-line formats requires a lot of time-consuming parsing work. As well as being a perl module, Log::Defer is also a specification for a structured logging format.
+Unfortunately, doing analysis on ad-hoc unstructured logs requires lots of menial coding work writing parsers. Even more annoying is that these parsers are often regexp-based and brittle.
 
-Although this module doesn't impose any external encoding for log messages on you, some tools like the visualisation tool only support JSON at this time.
+As well as being a perl module, Log::Defer is also a specification for a structured logging format. Although it doesn't impose any external encoding for log messages on you, some tools like the visualisation tool L<log-defer-viz> only support JSON at this time.
 
-The currently recommended format to store logs in is newline-separated, minified JSON. The newline is useful because it allows simple whole-request greping of the logs. With structured logs, much more accurate and flexible greping is possible, as is shown in L<log-defer-viz>.
+The currently recommended format to store logs in is newline-separated, minified JSON. The newline+minification is useful because it allows simple whole-request greping of the logs. With structured logs, much more accurate and flexible greping is also possible, as described in L<log-defer-viz>.
 
 
 
 =head1 LOG MESSAGES
 
-Log::Defer objects provide a very basic "log level" system. In order of increasing verbosity, here are the normal logging methods and their numeric log level:
+Log::Defer objects provide a very basic "log level" system. In order of increasing verbosity, here are the normal logging methods and their numeric log levels:
 
     $logger->error("...");  # 10
     $logger->warn("...");   # 20
@@ -300,17 +309,17 @@ You can also use custom log levels:
 
 If you pass in a C<verbosity> argument to the Log::Defer constructor, messages with a higher log level will not be included in the final log message. Otherwise, all log messages are included.
 
-Even if you include noisy debug logs you can filter them out with the visualisation tool at display time. The C<verbosity> argument is only useful for reducing the size of log messages or eliminating unnecessary processing overhead (see the no-overhead debug logs section below).
+Even if you record noisy debug logs you can filter them out with a visualisation tool at display time. The C<verbosity> argument is only useful for reducing the size of log messages or eliminating unnecessary processing overhead (see the no-overhead debug logs section below).
 
-Note that you can pass in multiple items to a log message and they don't even need to be strings:
+Note that you can pass in multiple items to a log message and they don't even need to be strings (but make sure you are catching any exceptions thrown by your encoder as done in the synopsis):
 
-    $logger->warn("something weird happened: $@", { username => $username });
+    $logger->error("peer timeout", { waited => $timeout });
 
-In the deferred logging callback, the log messages are recorded in the C<logs> element of the C<$msg> hash. It is an array ref and here would be the element pushed onto C<logs> by the C<warn> method call above:
+In the deferred logging callback, the log messages are recorded in the C<logs> element of the C<$msg> hash. It is an array ref and here is the element that would be pushed onto C<logs> by the C<error> method call above:
 
-    [ 0.201223, 20, "something weird happened: peer timeout", { username => "jimmy" } ]
+    [ 30.201223, 10, "peer timeout", { waited => 30 } ]
 
-The first element is a timestamp of how long the C<warn> method was called after the C<start> in seconds (see L<TIMERS> below). The second element is the verbosity level of this message.
+The first element is a timestamp of how long the C<error> method was called after the C<start> in seconds (see L<TIMERS> below). The second element is the verbosity level of this message. The remaining elements are passed in untouched from the C<error> method.
 
 
 
@@ -330,13 +339,13 @@ Instead of log messages, you can directly add items to a C<data> hash reference
 
     $log->data->{ip} = $ENV{REMOTE_ADDR};
 
-This is a useful place to recording info that needs to be extracted since you don't need to crawl through log message entries. Anything you put in the C<data> hash reference will be passed along untouched to your defered callback.
+This is a useful place to record info that needs to be extracted programatically. Anything you put in the C<data> hash reference will be passed along untouched to your defered callback (but again, make sure you are catching encoder exceptions as shown in the synopsis).
 
 
 
 =head1 TIMERS
 
-When the logger object is first created, the current time is recorded as a L<Time::HiRes> absolute timestamp and is stored in the C<start> element of the log hash. All other times are relative offsets from C<start>.
+When the logger object is first created, the current time is recorded as a L<Time::HiRes> absolute timestamp and is stored in the C<start> element of the log hash. C<start> is a L<Time::HiRes> absolute timestamp. All other times are relative offsets from C<start> in seconds.
 
 When the logger object is destroyed, the time elapsed since C<start> is stored in C<end>.
 
@@ -344,47 +353,12 @@ In addition to start and duration of the entire transaction, you can also record
 
 Timer objects are created by calling the C<timer> method on the logger object. This method should be passed a description of what you are timing.
 
-The timer starts as soon as the timer object is created and stops once the last reference to the timer is destroyed or goes out of scope. If the logger object itself is destroyed or goes out of scope then all outstanding timers are terminated at that point.
-
-C<start> is a L<Time::HiRes> absolute timestamp. All other times are relative offsets from this C<start> time. Everything is in seconds.
-
-Here is a fairly complicated example showing how to use concurrent timers:
-
-    sub handle_request {
-      my $request = shift;
-      my $logger = Log::Defer->new(\&my_logging_function);
-
-      my $headers = do {
-        my $parse_timer = $logger->timer('parsing request');
-        parse_request($request);
-      };
-
-      my $fetch_timer = $logger->timer('fetching results');
-      async_fetch_results($headers, sub {
-
-        ## stop first timer by undefing ref, then start new timer
-        undef $fetch_timer; $fetch_timer = $logger->timer('fetching stage 2');
-
-        async_fetch_stage_2($headers, sub {
-
-          $logger; ## keep reference alive
-          undef $fetch_timer;
-          send_response();
-
-        });
-
-        my $update_cache_timer = $logger->timer('update cache');
-
-        async_update_cach(sub {
-
-          $logger; ## keep reference alive
-          undef $update_cache_timer;
-
-        });
-
-      });
-    }
+The timer starts as soon as the timer object is created and stops once the last reference to the timer is destroyed or goes out of scope. If the logger object itself is destroyed or goes out of scope then all outstanding timers are terminated at that point:
 
+    {
+        my $timer = $log_defer_object->timer('running some_code');
+        some_code();
+    } ## <- timer is stopped here because $timer goes out of scope
 
 
 
@@ -476,9 +450,9 @@ L<Log::Defer github repo|https://github.com/hoytech/Log-Defer>
 
 One way to visualize logs created by this module is with the command-line script L<log-defer-viz>
 
-As mentioned above, this module doesn't actually log messages to disk/syslog/anything so you still must use some other module to record your log messages. There are many libraries on CPAN that can do this and there should be at least one that fits your requirements. Some examples are: L<Sys::Syslog>, L<Log::Dispatch>, L<Log::Handler>, L<Log::Log4perl>, L<Log::Fast>, L<AnyEvent::Log>.
+As mentioned above, this module doesn't itself log messages to disk so you still must use some other module to record your log messages. There are many libraries on CPAN that can do this and there should be at least one that fits your requirements. Some examples are: L<Sys::Syslog>, L<Log::Dispatch>, L<Log::Handler>, L<Log::Log4perl>, L<Log::Fast>, L<AnyEvent::Log>.
 
-Additionally, this module doesn't provide any official serialization format. There are many choices for this, including L<JSON::XS>, L<Sereal>, L<Storable>, and L<Data::MessagePack>.
+Additionally, this module doesn't provide any official serialization format. There are many choices for this, including L<JSON::XS> (JSON is the only format currently supported by L<log-defer-viz>), L<Sereal>, L<Storable>, and L<Data::MessagePack>.
 
 Currently the timestamp generation system is hard-coded to C<Time::HiRes::time>. You should be aware of some caveats related to non-monotonous clocks that are discussed in L<Time::HiRes>.
 
